/*----------------------------------------------------------------*/
/*                                                                */
/*   Turbo Vision 1.0                                             */
/*   Copyright (c) 1991 by Borland International                  */
/*                                                                */
/*   Turbo Vision TVTXTDMO source file                            */
/*      Modified by: Michael Shunfenthal 8/25/92                  */
/*                                                                */
/*    Displays an error message if the program is not invoked     */
/*    correctly: Syntax: TVTXTDM2 <file to view>                  */
/*    The version supplied with the compiler clears the screen    */
/*    immediately after the message is displayed, so it is not    */
/*    seen.                                                       */
/*                                                                */
/*    Notes:                                                      */
/*    The idea for this modification comes from TVEXIT in Lib 2   */
/*    The original version of checkParamList was passed the       */
/*    parameters argc and argv, where it checked for proper       */
/*    command line values.                                        */
/*    checkParamList had to be modified because it is defined to  */
/*    be exited by '#pragma exit checkParamList 31' (line ~164).  */
/*    However, such a function MUST be declared 'void func(void),'*/ 
/*    so the global variable exit_flag must be set in the TMyApp  */
/*    constructor, where argc is accessible.                      */
/*----------------------------------------------------------------*/

#define Uses_TWindow
#define Uses_TApplication
#define Uses_TRect
#define Uses_TTerminal
#define Uses_MsgBox
#define Uses_otstream
#define Uses_TDeskTop
#include <tv.h>

#include <string.h>
#include <stdlib.h>
#include <io.h>
#include <alloc.h>
#include <fcntl.h>
#include <strstrea.h>
#include <fstream.h>

// added to abort clearing screen
int exit_flag = 1;

class TTerminalWindow : public TWindow
{

public:

    TTerminalWindow( TRect bounds,
                     const char *winTitle,
                     ushort windowNo,
                     TTerminal *&interior,
                     ushort aBufSize
                   );
    TTerminal *makeInterior( TRect bounds, ushort aBufSize );
};

class TMyApp : public TApplication
{

public:

    TMyApp( int argc, char *argv[] );
    void showTerminalWindow( char *fileName );
};

void checkParamList( void )
{
    if( exit_flag == 2 )
        {
        cerr << "Syntax: TVTXTDMO <file to view>" << endl;
        exit (EXIT_FAILURE);
        }

    if( exit_flag == 1 ) // check for read permission
        {
        cerr << "Cannot open file" << endl;
        exit (EXIT_FAILURE);
        }
}

TTerminalWindow::TTerminalWindow( TRect bounds,
                                  const char *winTitle,
                                  ushort windowNo,
                                  TTerminal *&interior,
                                  ushort aBufSize
                                ) :
    TWindowInit( &TTerminalWindow::initFrame ),
    TWindow(bounds, winTitle, windowNo )
{
    interior = makeInterior( bounds, aBufSize );
    insert( interior );
}

TTerminal *TTerminalWindow::makeInterior( TRect bounds, ushort aBufSize )
{
    bounds = getExtent();
    bounds.grow( -1, -1 );
    return new TTerminal( bounds,
        0,			 // no horizontal scrollbar
        standardScrollBar( sbVertical | sbHandleKeyboard ),
        aBufSize );
};

TMyApp::TMyApp( int argc, char *argv[] ) :
    TProgInit( &TMyApp::initStatusLine,
               &TMyApp::initMenuBar,
               &TMyApp::initDeskTop
             )
{
/* argc must at least be greater than one to have the input filename as
   one parameter */
    if( argc < 2 )
        exit_flag = 2;
    else if( access( argv[1], 4 ) != 0 ) // check for read permission
        exit_flag = 1;
    checkParamList( );
    showTerminalWindow( argv[1] );
};

void TMyApp::showTerminalWindow( char *fileName )
{
    int handle = open( fileName, O_RDONLY );
    unsigned buffSize = 8192;
    if( coreleft() - 1024 < buffSize )
        buffSize = unsigned(coreleft() - 1024);

    if( filelength( handle ) > buffSize )
        {
        char buf[128];
        ostrstream os( buf, sizeof buf );
        os << "File is too big to fit in a TTerminal buffer. "
              "Only the first " << buffSize << " bytes of the file "
              "will be displayed." << ends;
        messageBox( buf, mfOKButton | mfWarning );
        }
    else
        buffSize = unsigned(filelength( handle ));
    close( handle );

    // Initialize the terminal window object
    TTerminal *interior;
    TTerminalWindow *demo = new TTerminalWindow( TRect( 10, 1, 70, 18 ),
                                                 fileName,
                                                 wnNoNumber,
                                                 interior,
                                                 buffSize
                                               );
    deskTop->insert( demo );

    // Assign the TTerminal interior text device driver to a text "file"
    otstream os( interior );
    
    ifstream is( fileName );
    is.unsetf( ios::skipws );
    char st[128];

    while( is.getline( st, sizeof st ) && interior->canInsert(strlen(st)) )
        os << st << endl;

    interior->scrollTo(0, 0);
};

//function constructed before and destructed after the TScreen object
#pragma exit checkParamList 31

int main( int argc, char *argv[] )
{
    TMyApp app( argc, argv );
    app.run();
    exit_flag = 0;
    return 0;
}
